package fr.soleil.docking.view;

import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.Action;

/**
 * A factory that takes care of initializing dockable components and transmitting models.
 * 
 * @author Hardion
 * @author GIRARDOT
 */
public interface IViewFactory {

    public static final String VIEWS = "views";

    /**
     * Returns the DynamicView associated with this id (creates the
     * DynamicView).
     * 
     * @param id
     *            The id of the desired DynamicView
     * @return The DynamicView associated with this id
     */
    public IView getView(Object id);

    public Collection<IView> getViews();

    /**
     * Returns the ActionList
     * 
     * @param id
     *            The id of the desired DynamicView
     * @return The DynamicView associated with this id
     */
    public List<Action> getActionList();

    /**
     * Saves preferences
     * 
     * @param prefs preferences in which to save
     */
    public void savePreferences(Preferences prefs);

    /**
     * Loads preferences
     * 
     * @param prefs preferences to load
     */
    public void loadPreferences(Preferences prefs);

    public int size();

    /**
     * @param listener
     * @see java.beans.PropertyChangeSupport#addPropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public void addPropertyChangeListener(PropertyChangeListener listener);

    /**
     * @param listener
     * @see java.beans.PropertyChangeSupport#removePropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public void removePropertyChangeListener(PropertyChangeListener listener);

    public Object readViewId(ObjectInputStream in) throws IOException;

    public void writeViewId(Object id, ObjectOutputStream out) throws IOException;

    public IView addView(IView view);

    public boolean removeView(IView view);

    /**
     * Removes an {@link IView}, determined by an id, from list and from the
     * docking area.
     * 
     * @param id The id of the {@link IView} to remove.
     * @return The removed {@link IView} if such an {@link IView} existed and
     *         was successfully removed, <code>null</code> otherwise.
     */
    public IView removeView(Object id);
}
